REBOL [
	Title: "REBOL/View Desktop - Icon Handling"
	Version: 1.0.0
	Author: "Carl Sassenrath"
	Rights: "Copyright REBOL Technologies 1999-2005. All rights reserved."
	; You are free to use, modify, and distribute this software with any
	; REBOL Technologies products as long as the above header, copyright,
	; and this comment remain intact. This software is provided "as is"
	; and without warranties of any kind. In no event shall the owners or
	; contributors be liable for any damages of any kind, even if advised
	; of the possibility of such damage. See license for more information.

	; Please help us to improve this software by contributing changes and
	; fixes via http://www.rebol.com/feedback.html - Thanks!
]

;=== Icon Functions =========================================================

icon-space: 6		; spacing between icons (could be made a preference someday)
dtf-slide-max: 10
image-cache: []		; block of desktop image files loaded this session

make-icon: func [ico /local icon text-height][
	; Make an icon from the default icon face.
	; Set its color and load its image, using a default image based on the
	; icon file suffix (type).
	icon: make-face dtc-icon
	icon/user-data: ico
	icon/text: ico/name

	if (length? ico/name) > 32 [
		icon/text: append copy/part ico/name 32 "..."
	]
	icon/font/valign: 'bottom
	icon/para/wrap?: true
	icon/para/origin: 0x48

	if ico/color [icon/font: make icon/font [color: ico/color]]

	icon/pane: make icon/pane [
		;print [ico/name ico/type]
		if all [word? ico/image url? user-prefs/desktop-url] [
			ico/image: rejoin [dirize user-prefs/desktop-url "icons/" ico/image ".gif"] ; allow png too!
		]
		image: load-safe-image ico/image
		if not image [
			image: case [
				ico/type = 'folder [folder-icon]
				ico/type = 'link [load-desktop-icon 'html]
				ico/type = 'email [load-desktop-icon 'email]
				email? ico/item [load-desktop-icon 'email]
				true [load-default-icon ico/item]
			]
		]
		if not image [image: default-icon]
		feel: none
		color: none
		effect: any [ico/effect [key 174.154.122 shadow smooth]]
	]

	; Make icon proper size:
	if icon/pane/image [
		icon/pane/size: 64x48
		icon/pane/offset: 0x0
		if not outside? icon/pane/size icon/pane/image/size [
			icon/pane/size: icon/pane/image/size
			icon/pane/offset: 64x48 - icon/pane/image/size / 2
			if not ico/edge [
				icon/pane/edge/effect: none
				icon/pane/edge/size: none
			]
		]
	]

	; Determine width of the icon label:
	text-height: second size-text make face [size: 70x250 text: icon/text  font: icon/font]
	icon/size/y: 50 + text-height

	icon
]

load-safe-image: func [
	; Load an icon from a file or URL without choking on it:
	file "Local file or remote URL"
	/update "Force update from source site"
	/clear "Purge the entire cache"
	/local image
][
	if not any [file? file url? file] [return none]
	if clear [system/words/clear image-cache recycle]
	file: full-path file
	if any [update  not image: select image-cache file] [
		if all [update image] [remove/part find image-cache file 2]
		image: load-binary file not update
		repend image-cache [file image]
	]
	all [image? image image] ; returns file data
]

to-icon-path: func [name [word!]] [
	; Convert a file's datatype (word) to a loadable image name:
	join view-root/desktop/icons ["/" name ".gif"]
]

load-desktop-icon: func [file-type /local data url][ ; file-type is a word
	if file-type = 'rebol [return reblet-icon]
	data: load-safe-image to-icon-path file-type
	if not data [
		url: dirize any [user-prefs/desktop-url http://www.rebol.com/view/icons/desktop]
		data: load-safe-image rejoin [url "icons/" file-type ".gif"]
		if data [write/binary to-icon-path file-type data]
	]
	data
]

load-default-icon: func [file /local n][
	; Load the default icon for a file:
	if all [
		any [file? file url? file]
		file: attempt [second split-path file]
		n: map-suffix file
	][
		load-desktop-icon n
	]
	;return it or none
]

layout-icons: function [icons] [xy maxy][
	; Layout icons in a pane:
	sort/compare icons func [a b] [(form a/text) < (form b/text)] ;>
	xy: dtc-icon/offset
	maxy: 0 ; tallest icon rules
	foreach icon icons [
		if icon/style = 'icon [
			if (first xy + icon/size) > dtc-face/size/x [
				xy: (xy + maxy + icon-space * 0x1) + (dtc-icon/offset * 1x0)
				maxy: 0
			]
			icon/offset: xy
			xy: icon/size * 1x0 + xy + (icon-space * 1x0)
			maxy: max maxy icon/size/y
		]
	]
	dtc-face/size/y: dtf-slide-max: xy/y + maxy
	dtf-slide/data: 0.0
	dtc-face/offset/y: 0
	remove find dtf-face/pane dtf-slide
	if dtf-slide-max > dtf-icons/size/y [
		append dtf-face/pane dtf-slide
		dtf-slide/redrag dtf-icons/size/y / dtf-slide-max
	]
]

dtc-icon/feel: make dtc-icon/feel [
	; Specify the event processing used for icons:

	over: func [f a pos of /local str][
		either a [ 
			f/font/color: dtw-skin/over-text-color
			if f/user-data/color [f/font/color: complement f/user-data/color]
			if f/user-data/over-color [f/font/color: f/user-data/over-color]
			show f
			f/font/color: any [f/user-data/color dtw-skin/base-text-color black] ;white
			if f/user-data/info [tell f/user-data/info]
		][
			show f
			tell ""
		]
	]

	engage: func [face action event /local icon][
		icon: face/user-data
		switch action [
			down [face/state: on]
			alt-down [face/state: on]
			up   [
				if face/state [
					if none? icon/type [icon/type: 'file] ; default
					dbug ["Icon do" third icon]
					switch icon/type [
						file   [do-file icon/item any [icon/stats true]]
						link   [if icon/item [browse full-path icon/item]]
						folder [show-folder icon/item]
					]
				]
				face/state: off
			]
			alt-up [
				dbug ["Icon info" third icon]
				if all [face/state icon] [show-icon-info icon]
				face/state: off
			]
			over [face/state: on]
			away [face/state: off]
		]
		cue face action
		show face
	]
]

